/* 
 * Copyright (C) 2006, iRex Technologies B.V.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * Portions of this code are taken from xpdf and are
 * Copyright 1996-2003 Glyph & Cog, LLC
 */

#include "XMgr.h"
#include "goo/gmem.h"
#include "PDFViewerLog.h"
#include <liberdm/display.h>
#define defaultRGBCube  5

//////////////////////////////////////////////////////////////////////////
static String fallbackResources[] = {
  NULL
};

static XrmOptionDescRec xOpts[] = {
};

#define nXOpts (sizeof(xOpts) / sizeof(XrmOptionDescRec))

struct XPDFAppResources 
{
    String geometry;
    String title;
    Bool installCmap;
    int rgbCubeSize;
    Bool reverseVideo;
    String paperColor;
    String matteColor;
    String initialZoom;
    Bool viKeys;
};

#define nXResources (sizeof(xResources) / sizeof(XtResource))

// these macro defines conflict with xpdf's Object class
#ifdef LESSTIF_VERSION
#undef XtDisplay
#undef XtScreen
#undef XtWindow
#undef XtParent
#undef XtIsRealized
#endif

XMgr::XMgr(void)
: display(NULL)
, screenNum(0)
, drawAreaGC(NULL)
, visual(NULL)
, rgbCubeSize(5)
, depth(8)
, mattePixel(Gulong(0x00ffffff))
{
}

XMgr::~XMgr(void)
{
    XtDestroyWidget(win);
}

void XMgr::setupX(int * argc, char * argv[])
{
    int n = 0;
    Arg args[20];

    // app shell
    appShell  = XtAppInitialize(&appContext, AppName, xOpts, nXOpts, 
                    argc, argv, fallbackResources, NULL, 0);
    display   = XtDisplay(appShell);
    screenNum = XScreenNumberOfScreen(XtScreen(appShell));
    visual    = DefaultVisual(display, screenNum);
    
    // initialize app shell 
    n = 0;
    XtSetArg(args[n], XmNwidth, SCREEN_WIDTH); ++n;
    XtSetArg(args[n], XmNheight, CLIENT_AREA); ++n;
    XtSetValues(appShell, args, n);

    // main window
    n = 0;
    XtSetArg(args[n], XmNminWidth, SCREEN_WIDTH); ++n;
    XtSetArg(args[n], XmNminHeight, CLIENT_AREA); ++n;
    XtSetArg(args[n], XmNbaseWidth, SCREEN_WIDTH); ++n;
    XtSetArg(args[n], XmNbaseHeight, CLIENT_AREA); ++n;
    XtSetArg(args[n], XmNdeleteResponse, XmDO_NOTHING); ++n;
    XtSetArg(args[n], XmNgeometry, "+0+0"); ++n;

    // should be win, the name should be same with registry
    win = XtCreatePopupShell("win", topLevelShellWidgetClass, 
            appShell, args, n);

    // draw area
    n = 0;
    XtSetArg(args[n], XmNresizePolicy, XmRESIZE_ANY); ++n;
    XtSetArg(args[n], XmNwidth, SCREEN_WIDTH); ++n;
    XtSetArg(args[n], XmNheight, CLIENT_AREA); ++n;
    XtSetArg(args[n], XmNbackground, 0x0ff); ++n;
    drawArea = XmCreateDrawingArea(win, "drawArea", args, n);
    XtManageChild(drawArea);   
    XtOverrideTranslations(drawArea, XtParseTranslationTable(
      "<Btn1Down>:DrawingAreaInput()\n"
      "<Btn1Up>:DrawingAreaInput()\n"
      "<Btn1Motion>:DrawingAreaInput()\n"
      "<Motion>:DrawingAreaInput()"));

    // popup window
    XtPopup(win, XtGrabNone); 
}

// We move the Grey Scale SplashBitmap into a memory buffer X can free and then
// create an XImage to wrap that buffer.
XImage * XMgr::createImage(SplashBitmap * bmp, 
                            int xSrc , int ySrc, 
                            GBool bRelease )
{
    if (NULL == bmp)
    {
        PV_ERRORPRINTF("bitmap is empty!");
        return NULL;
    }

    // Calculate size of the grey scale bitmap.
    // Pretty easy since it is one byte per pixel.
    size_t length = bmp->getHeight() * bmp->getWidth();

    // Allocate an image memory buffer X can free when it is ready
    char* buffer = (char *)gmalloc(length);

    // Check that we were able to get the memory we needed
    if (buffer == NULL) {
      PV_ERRORPRINTF("Couldn't allocate new image buffer");
      return NULL;
    }

    // Copy image data from SplashBitmap into new memory buffer
    memcpy( buffer, (char *)bmp->getDataPtr(), length);

    // Create XImage wrapper
    XImage *image = XCreateImage(display,
				 visual,
				 8,		// Depth of 8
				 ZPixmap, 
				 0,		// No skip at start of row
				 buffer,
				 bmp->getWidth(),
				 bmp->getHeight(),
				 8,		// 8 bits per pixel
				 0);		// Let X calculate row length
    if (NULL == image)
    {
        return NULL;
    }

    if (bRelease)
    {
        delete bmp;
    }
    return image;
}

void XMgr::drawImage(XImage * image, 
                          int xSrc , int ySrc , 
                          int xDest, int yDest, 
                          int width, int height )
{
    Window drawAreaWin;
    XGCValues gcValues;

    // create a GC for the drawing area
    drawAreaWin = XtWindow(drawArea);
    if (!drawAreaGC) 
    {
        gcValues.foreground = mattePixel;
        drawAreaGC = XCreateGC(display, drawAreaWin, GCForeground, &gcValues);
    }
    else
    {
        XSetForeground(display, drawAreaGC, mattePixel);
    }

    // draw the document
    XFillRectangle(display, drawAreaWin, drawAreaGC,
            0, 0, SCREEN_WIDTH, CLIENT_AREA);
            
    if (image) 
    {
        if (width < 0) width = image->width;
        if (height < 0) height = image->height;
        XPutImage(display, drawAreaWin, drawAreaGC, image,
            xSrc, ySrc, xDest, yDest, width, height);
            
        XSetForeground(display, drawAreaGC, 0);
        XDrawRectangle(display, drawAreaWin, drawAreaGC, xDest - 1, yDest - 1, width + 2, height + 2);
    } 
}
